# Usage: perl empegdir.pl [directory tree]
#
# EX: perl empegdir.pl c:\MP3\
#
# Outputs a listing of albums with their associated PIN numbers to be referenced
# by on an Empeg\RioCar player.
#
# Copyright 2002 by Colin "Yang" Verne
# Version 1.0a

use MP3::Info 0.91;

# read and parse mp3's in file tree
sub read_directory
{
  my ($dir, $filehash) = @_;
  my (@filelist, $tags, %tags, $file, $pin, $key);
  opendir(DIR, $dir) or return;

  @filelist = readdir(DIR);

  foreach $file (@filelist)
  {
    if (-f "$dir\\$file")
    {
      $tags = get_mp3tag("$dir\\$file");
      if ($$tags{"TITLE"})
      {
        $pin = lc($$tags{"TITLE"});
        $pin =~ tr/abcdefghijklmnopqrstuvwxyz0-9 !@#$%^&*()-=_+[]\\{}|;\':\",.\/<>?/222333444555666707788899900-9/d;
        $$filehash{$pin} = $tags;
      }
    }
    elsif (-d "$dir\\$file" && $file ne '.' && $file ne '..')
    {
      read_directory("$dir\\$file", $filehash);
    }
  }
  return;
}
#end read_directory()


my $startdir = shift(@ARGV);
$startdir = "." unless $startdir;
if ($startdir =~ /^(.*?)\\*$/)
{
  $startdir = $1;
}
unless (-d $startdir)
{
  print "Directory '$startdir' doesn't exist.";
  exit;
}

#read all of the mp3's in the filetree
%filehash = ();
read_directory($startdir, \%filehash);

#create a mapping of PIN prefixes
%prefixhash = ();
foreach $key (sort(keys(%filehash)))
{
  $filekey = $key;
  while ($key ne "")
  {
    if (defined($prefixhash{$key}))
    {
      $prefix = $prefixhash{$key};
      push @$prefix, $filehash{$filekey};
    }
    else
    {
      $prefixhash{$key} = [$filehash{$filekey}];
    }
    
    #strip off the last character
    $key = substr($key, 0, -1);
  }
}

#find the unambiguous keys
%finalhash = ();
foreach $key (sort(keys(%prefixhash)))
{
  $keylist = $prefixhash{$key};
  @keylist = @$keylist;
  if ($#keylist == 0)
  {
    $tags = shift(@keylist);
    $newkey = $$tags{"ARTIST"}." - ".$$tags{"ALBUM"}." - ".$$tags{"TITLE"};

    #if key is shorter
    if (!defined($$tags{"PIN"}) || length($key) < length($$tags{"PIN"}))
    {
      $$tags{"PIN"} = $key;
    }

    $finalhash{$newkey} = $tags;
  }
}

foreach $key (sort(keys(%finalhash)))
{
  $tags = $finalhash{$key};

  printf("%-70s %d\n",$key,$$tags{"PIN"});
}
